@@ -1,5 +1,6 @@
 from typing import Optional, List, Union
 import time
+import logging
 from hein_robots.robotics import Location, Twist, Wrench, Cartesian, Orientation, Units, Frame
 from hein_robots.base import robot_arms
 from hein_robots.kinova.kinova_gripper import KinovaGripper
@@ -33,6 +34,7 @@ class KinovaGen3Arm(robot_arms.RobotArm):
 
     @position_units.setter
     def position_units(self, value: str):
+        self.logger.debug(f'position_units = {value}')
         self._position_units = value
 
     @property
@@ -58,6 +60,7 @@ class KinovaGen3Arm(robot_arms.RobotArm):
 
     @max_velocity.setter
     def max_velocity(self, value: float):
+        self.logger.debug(f'max_velocity = {value}')
         self._max_velocity = value
 
     @property
@@ -69,6 +72,7 @@ class KinovaGen3Arm(robot_arms.RobotArm):
         if value < 0 or value > self.max_velocity:
             raise robot_arms.RobotArmInvalidVelocityError(f'Invalid velocity: {value} m/s, must be less than {self.max_velocity} m/s')
 
+        self.logger.debug(f'default_velocity = {value}')
         self._default_velocity = value
 
     @property
@@ -122,6 +126,10 @@ class KinovaGen3Arm(robot_arms.RobotArm):
         self._joint_count = self.connection.client.GetActuatorCount().count
         return self._joint_count
 
+    @property
+    def gripper_joint(self) -> int:
+        return self.joint_count - 1
+
     @property
     def gripper_position(self) -> float:
         return self.gripper.position
@@ -141,6 +149,8 @@ class KinovaGen3Arm(robot_arms.RobotArm):
 
     @tool_offset.setter
     def tool_offset(self, value: Location):
+        self.logger.debug(f'tool_offset = {value}')
+
         offset_m = value.convert_mm_to_m()
         tool_configuration = self.tool_configuration
         tool_configuration.tool_transform.x = offset_m.x
@@ -157,6 +167,8 @@ class KinovaGen3Arm(robot_arms.RobotArm):
 
     @tool_mass.setter
     def tool_mass(self, value: float):
+        self.logger.debug(f'tool_mass = {value}')
+
         tool_configuration = self.tool_configuration
         tool_configuration.tool_mass = value
         self.connection.control_client.SetToolConfiguration(tool_configuration)
@@ -169,6 +181,8 @@ class KinovaGen3Arm(robot_arms.RobotArm):
         self.connection.disconnect()
 
     def set_servo_mode(self, mode: int):
+        self.logger.debug(f'set_servo_mode(mode={mode})')
+
         servo_mode = Base_pb2.ServoingModeInformation()
         servo_mode.servoing_mode = mode
         self.connection.client.SetServoingMode(servo_mode)
@@ -192,6 +206,8 @@ class KinovaGen3Arm(robot_arms.RobotArm):
         self.connection.execute_existing_action('Home', Base_pb2.REACH_JOINT_ANGLES, wait=wait)
 
     def set_tool(self, offset: Location, mass_kg: float):
+        self.logger.debug(f'set_tool(offset={offset}, mass_kg={mass_kg}')
+
         tool_transform = ControlConfig_pb2.CartesianTransform()
         tool_transform.x = offset.x
         tool_transform.y = offset.y
@@ -274,6 +290,9 @@ class KinovaGen3Arm(robot_arms.RobotArm):
         if frame is not None:
             raise robot_arms.RobotArmNotSupportedError(f'Frame parameter not supported')
 
+        self.logger.debug(f'move_to_location({location}, velocity={velocity}, acceleration={acceleration}, frame={frame}, '
+                          f'relative={relative}, wait={wait}, timeout={timeout})')
+
         action = Base_pb2.Action()
 
         target_pose = action.reach_pose.target_pose
@@ -309,6 +328,8 @@ class KinovaGen3Arm(robot_arms.RobotArm):
         if len(joint_positions) != self.joint_count:
             raise robot_arms.RobotArmInvalidJointsError(f'Invalid number of joint angles ({len(joint_positions)}), must be {self.joint_count}')
 
+        self.logger.debug(f'move_joints({joint_positions}, velocity={velocity}, acceleration={acceleration}, relative={relative}, wait={wait}, timeout={timeout})')
+
         action = Base_pb2.Action()
 
         joint_offsets = self.joint_positions if relative else [0] * self.joint_count
@@ -327,6 +348,8 @@ class KinovaGen3Arm(robot_arms.RobotArm):
     def move_joint(self, joint_id: int, position: float,
                       velocity: Optional[float] = None, acceleration: Optional[float] = None,
                       relative: bool = False, wait: bool = True, timeout: Optional[float] = None):
+        self.logger.debug(f'move_joint({joint_id}, {position}, velocity={velocity}, acceleration={acceleration}, relative={relative}, wait={wait}, timeout={timeout})')
+
         action = Base_pb2.Action()
 
         joint_offset = self.joint_positions[joint_id] if relative else 0
@@ -342,6 +365,7 @@ class KinovaGen3Arm(robot_arms.RobotArm):
 
     def move_twist(self, x: float = 0.0, y: float = 0.0, z: float = 0.0, rx: float = 0.0, ry: float = 0.0, rz: float = 0.0,
                    duration: Optional[float] = None, wait: bool = True, timeout: Optional[float] = None):
+        self.logger.debug(f'move_twist({x}, {y}, {z}, {rx}, {ry}, {rz}, duration={duration}, wait={wait}, timeout={timeout})')
         action = Base_pb2.Action()
         twist_command = action.send_twist_command
         twist_command.reference_frame = Base_pb2.CARTESIAN_REFERENCE_FRAME_TOOL
@@ -371,4 +395,6 @@ class KinovaGen3Arm(robot_arms.RobotArm):
         elif isinstance(position, bool):
             position = 0.0 if position else 1.0
 
+        self.logger.debug(f'open_gripper({position}, {force}, {velocity}, wait={wait}, timeout={timeout}')
+
         self.gripper.move(position, force=force or self.gripper_default_force, velocity=velocity or self.gripper_default_velocity, timeout=timeout or 10.0)
